/* 
 * Copyright 2013 ScaleOut Software, Inc.
 * 
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

package com.scaleoutsoftware.demos.alertsdesk;

import com.scaleoutsoftware.soss.client.*;
import com.scaleoutsoftware.demos.common.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultListModel;
import javax.swing.RowSorter;
import javax.swing.SortOrder;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.table.TableColumn;
import javax.swing.table.TableRowSorter;

/**
 * A Swing UI form that displays the alert status of all strategies in the ScaleOut in-memory
 * data grid, as well as the alert status for individual positions within each strategy.
 * 
 * Also allow for the configuration of the alert threshold that is used by all price update workers.
 */
public class AlertForm extends javax.swing.JFrame {

    private final int REFRESH_INTERVAL_SEC = 2;
    
    // Scheduled task that refreshes the display of strategies and their current alert status:
    private final ScheduledExecutorService refreshScheduler;
    
    // Model used by the JList of strategies:
    private DefaultListModel<StrategyListItem> listModel = new DefaultListModel<>();
    
    // Model used by the JTable of a strategy's positions:
    private PositionTableModel tableModel = new PositionTableModel();
    
    // Latest alert threshold percentage that is stored in the ScaleOut in-memory data grid.
    private volatile BigDecimal latestThreshold = BigDecimal.ZERO;
    
    private volatile int lastStrategyCount = 0;

    private volatile boolean repopulateStrategyList = false;
    
    /**
     * Creates new instance of the form.
     */
    public AlertForm() {
        this.refreshScheduler = Executors.newScheduledThreadPool(1);
        initComponents();
        
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jSplitPane3 = new javax.swing.JSplitPane();
        jSplitPane4 = new javax.swing.JSplitPane();
        jScrollPane1 = new javax.swing.JScrollPane();
        strategyList = new javax.swing.JList();
        jScrollPane2 = new javax.swing.JScrollPane();
        positionTable = new javax.swing.JTable();
        jScrollPane4 = new javax.swing.JScrollPane();
        jPanel2 = new javax.swing.JPanel();
        jLabel3 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        lblPosEval = new javax.swing.JLabel();
        lblThresholdVal = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        lblThroughputVal = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        mnuExit = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        mnuConfigThreshold = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Strategy Alerts");
        setLocationByPlatform(true);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowOpened(java.awt.event.WindowEvent evt) {
                onWindowOpened(evt);
            }
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jSplitPane3.setDividerLocation(40);
        jSplitPane3.setOrientation(javax.swing.JSplitPane.VERTICAL_SPLIT);

        jSplitPane4.setBorder(null);
        jSplitPane4.setDividerLocation(120);

        strategyList.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        strategyList.setToolTipText("");
        strategyList.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                strategyListValueChanged(evt);
            }
        });
        jScrollPane1.setViewportView(strategyList);

        jSplitPane4.setLeftComponent(jScrollPane1);

        positionTable.setAutoCreateColumnsFromModel(false);
        positionTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        positionTable.setRowSelectionAllowed(false);
        jScrollPane2.setViewportView(positionTable);

        jSplitPane4.setRightComponent(jScrollPane2);

        jSplitPane3.setRightComponent(jSplitPane4);

        jScrollPane4.setHorizontalScrollBarPolicy(javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
        jScrollPane4.setVerticalScrollBarPolicy(javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);

        jLabel3.setText("Alert Threshold:");

        jLabel1.setText("Positions Evaluated:");

        lblPosEval.setText("N/A");

        lblThresholdVal.setText("N/A");

        jLabel2.setText("Throughput (pos/sec):");

        lblThroughputVal.setText("N/A");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(lblThresholdVal)
                .addGap(18, 18, 18)
                .addComponent(jLabel1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(lblPosEval)
                .addGap(18, 18, 18)
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(lblThroughputVal)
                .addContainerGap(477, Short.MAX_VALUE))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel3)
                    .addComponent(jLabel1)
                    .addComponent(lblPosEval)
                    .addComponent(lblThresholdVal)
                    .addComponent(jLabel2)
                    .addComponent(lblThroughputVal))
                .addContainerGap(304, Short.MAX_VALUE))
        );

        jScrollPane4.setViewportView(jPanel2);

        jSplitPane3.setLeftComponent(jScrollPane4);

        jMenu1.setText("File");

        mnuExit.setText("Exit");
        mnuExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuExitActionPerformed(evt);
            }
        });
        jMenu1.add(mnuExit);

        jMenuBar1.add(jMenu1);

        jMenu2.setText("Tools");

        mnuConfigThreshold.setText("Configure alert threshold...");
        mnuConfigThreshold.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                mnuConfigThresholdActionPerformed(evt);
            }
        });
        jMenu2.add(mnuConfigThreshold);

        jMenuBar1.add(jMenu2);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jSplitPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 657, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jSplitPane3, javax.swing.GroupLayout.DEFAULT_SIZE, 342, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void onWindowOpened(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_onWindowOpened
        
        // Configure the strategy list control and perform initial population. Using our
        // own renderer here so that we can perform highlighting on alerted strategies:
        strategyList.setCellRenderer(new StrategyListRenderer());
        populateStrategyList();
        
        // Configure the position table control:
        String[] positionTableColumns = new String[]  {"Position Type", "Ticker", "Price", "Position", "Exposure", "Target Allocation (%)", "Actual Allocation (%)", "Deviation (%)"};
        for (int i = 0; i < positionTableColumns.length; i++)
        {
            TableColumn col = new TableColumn(i);
            col.setHeaderValue(positionTableColumns[i]);
            positionTable.addColumn(col);
        }
        // Wire in our custom model and renderer (handles highlighting) for the position table:
        positionTable.setModel(tableModel);
        tableModel.init();
        positionTable.setDefaultRenderer(Object.class, new PositionTableRenderer());
        
        // Set up sorting on the position table:
        // TODO: enhance sorting so that numeric values are sorted properly (not alphabetically)
        TableRowSorter<PositionTableModel> sorter = new TableRowSorter<>(tableModel);
        positionTable.setRowSorter(sorter);
        // Initial sort is by position type (core/hedge), then by ticker symbol:
        List <RowSorter.SortKey> sortKeys = new ArrayList<>();
        sortKeys.add(new RowSorter.SortKey(0, SortOrder.ASCENDING));
        sortKeys.add(new RowSorter.SortKey(1, SortOrder.ASCENDING));
        sorter.setSortKeys(sortKeys);
        
        // Start the automatic refresh for the strategy list:
        refreshScheduler.scheduleWithFixedDelay(new Runnable() {public void run() {refreshStrategies();}},
                                                REFRESH_INTERVAL_SEC, REFRESH_INTERVAL_SEC, TimeUnit.SECONDS);
        
    }//GEN-LAST:event_onWindowOpened

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        tableModel.close();
        refreshScheduler.shutdown();
    }//GEN-LAST:event_formWindowClosing

    private void strategyListValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_strategyListValueChanged
        // the user selected a different strategy in the JList on the left. Update
        // the table of positions to display the selected strategy's positions.
        tableModel.changeStrategy(strategyList.getSelectedIndex());
        tableModel.fireTableDataChanged();
    }//GEN-LAST:event_strategyListValueChanged

    private void mnuExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuExitActionPerformed
        System.exit(0);
    }//GEN-LAST:event_mnuExitActionPerformed

    private void mnuConfigThresholdActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_mnuConfigThresholdActionPerformed
        // User wants to change the alert threshold that's stored in the settings object we keep
        // in the IMDG. Show a dialog that performs the change:
        ConfigThresholdDialog threshDlg = new ConfigThresholdDialog(this, true, latestThreshold);
        threshDlg.setVisible(true);
    }//GEN-LAST:event_mnuConfigThresholdActionPerformed

    private void populateStrategyList()
    {
        // Perform initial population of the JList of strategy items. Rather than reading every
        // single strategy from the cache, we can generate a list of strategies items from the latest
        // alert summary that's stored in the in-memory data grid.
        try {
            listModel.clear();

            NamedCache summaryCache = CacheFactory.getCache(Settings.SUMMARY_CACHE_NAME);
            AlertSummary alertSummary = (AlertSummary) summaryCache.get(Settings.SUMMARY_OBJ_NAME);
            if (alertSummary == null) {
                // User probably hasn't started the market simulator app yet. Nothing to do.
                return;
            }

            for (int i = 0; i < alertSummary.getTotalStrategyCount(); i++) {
                listModel.addElement(new StrategyListItem(i, alertSummary.getAlertedStrategies().get(i)));
            }

            strategyList.setModel(listModel);
        } catch (NamedCacheException ex) {
            Logger.getLogger(AlertForm.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    private void refreshStrategies()
    {
        // Refreshes the UI with the latest alert statuses:
        try {
            NamedCache summaryCache = CacheFactory.getCache(Settings.SUMMARY_CACHE_NAME);
            final AlertSummary alertSummary = (AlertSummary)summaryCache.get(Settings.SUMMARY_OBJ_NAME);
            
            if (alertSummary == null)
            {
                // User probably hasn't started the market simulator app yet. Nothing to do.
                return;
            }

            if (alertSummary.getTotalStrategyCount() != lastStrategyCount)
            {
                // The number of strategies has changes since we last refreshed. Reinitialize
                // the strategy list.
                repopulateStrategyList = true;
            }
            else
            {
                repopulateStrategyList = false;
            }


            final int positionsEvaluated = alertSummary.getTotalPositionsEvaluated();
            for (int i = 0; i < alertSummary.getTotalStrategyCount(); i++)
            {
                listModel.get(i).setAlerted(alertSummary.getAlertedStrategies().get(i));
            }

            
            final BigDecimal alertThreshold = retrieveAlertThreshold();
            latestThreshold = alertThreshold;
            
            double elapsedTimeSec = (double)alertSummary.getElapsedEvaluationTime() / 1000.0;
            final double throughput = (double)alertSummary.getTotalPositionsEvaluated() / elapsedTimeSec;
            
            // post a repaint request to the UI's event thread--this will cause 
            // the list's highlights to be updated:
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    if(repopulateStrategyList) {
                        populateStrategyList();
                        lastStrategyCount = alertSummary.getTotalStrategyCount();
                    }
                    strategyList.repaint();
                    lblPosEval.setText(Integer.toString(positionsEvaluated));
                    lblThresholdVal.setText(alertThreshold.setScale(1, RoundingMode.HALF_UP).toString() + "%");
                    lblThroughputVal.setText(String.format("%.1f", throughput));
                }
            });
            
        } catch (NamedCacheException ex) {
            Logger.getLogger(AlertForm.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    private BigDecimal retrieveAlertThreshold() {
        // get the latest alert threshold summary being used by the price update workers:
        try {
            NamedCache settingsCache = CacheFactory.getCache(Settings.SETTINGS_CACHE_NAME);
            Settings settings = (Settings)settingsCache.get(Settings.SETTINGS_OBJ_NAME);
            if (settings != null)
                return settings.getMaxAllowedDeviationPercent();
            else
                return BigDecimal.ZERO;
        } catch (NamedCacheException ex) {
            Logger.getLogger(AlertForm.class.getName()).log(Level.SEVERE, null, ex);
            return BigDecimal.ZERO;
        }
    }
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* Go with the system look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            javax.swing.UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(AlertForm.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>
        
        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new AlertForm().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JScrollPane jScrollPane4;
    private javax.swing.JSplitPane jSplitPane3;
    private javax.swing.JSplitPane jSplitPane4;
    private javax.swing.JLabel lblPosEval;
    private javax.swing.JLabel lblThresholdVal;
    private javax.swing.JLabel lblThroughputVal;
    private javax.swing.JMenuItem mnuConfigThreshold;
    private javax.swing.JMenuItem mnuExit;
    private javax.swing.JTable positionTable;
    private javax.swing.JList strategyList;
    // End of variables declaration//GEN-END:variables
}
