/* 
 * Copyright 2013 ScaleOut Software, Inc.
 * 
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

package com.scaleoutsoftware.demos.alertsdesk;

import com.scaleoutsoftware.soss.client.*;
import com.scaleoutsoftware.demos.common.*;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.SwingUtilities;
import javax.swing.table.AbstractTableModel;

/**
 * Custom data model used by the JTable that displays a strategy's position data.
 * 
 * This model automatically refreshes its data every 2 seconds.
 */
public class PositionTableModel extends AbstractTableModel {
    private final int REFRESH_INTERVAL_SEC = 2;
    
    // Scheduled task that refreshes the display of positions and their current alert status:
    private final ScheduledExecutorService refreshScheduler;
    
    // ScaleOut named cache object containing strategy objects.
    private NamedCache strategyCache;
    
    // The strategy object whose positions are currently being displayed to the user.
    private Strategy strategy = null;
    

    /**
     * Constructor
     */
    public PositionTableModel()
    {
        // Set up the scheduled refesh task:
        this.refreshScheduler = Executors.newScheduledThreadPool(1);
         
        
        // Configure the ScaleOut named cache that holds all the strategies:
        try {
            strategyCache = CacheFactory.getCache(Settings.STRATEGY_CACHE_NAME);
        } catch (NamedCacheException ex) {
            Logger.getLogger(PositionTableModel.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    public void init()
    {
        refreshScheduler.scheduleWithFixedDelay(new Runnable() {public void run() {refreshPositions();}},
                                                REFRESH_INTERVAL_SEC, REFRESH_INTERVAL_SEC, TimeUnit.SECONDS);
    }
    
    public void close()
    {
        refreshScheduler.shutdown();
    }
    
    /**
     * Changes the strategy whose positions are to be displayed to the user.
     * @param strategyId identifier of the strategy
     */
    public synchronized void changeStrategy(int strategyId)
    {
        if (strategyCache == null)
            return;
        
        try {
            strategy = (Strategy)strategyCache.get(Integer.toString(strategyId));
        } catch (NamedCacheException ex) {
            Logger.getLogger(PositionTableModel.class.getName()).log(Level.SEVERE, null, ex);
            strategy = null;
        }
    }
    
    /**
     * Updates strategy whose positions are being displayed with the latest version being
     * held in the SOSS in-memory data grid.
     */
    public synchronized void refreshPositions()
    {
        if (strategyCache == null || strategy == null)
            return;
        
        try {
            strategy = (Strategy)strategyCache.get(Integer.toString(strategy.getId()));
        } catch (NamedCacheException ex) {
            Logger.getLogger(PositionTableModel.class.getName()).log(Level.SEVERE, null, ex);
            strategy = null;
            return;
        }
        
        // Tell the JTable that data in our model is changed:        
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                fireTableDataChanged();
            }
        });
    }
    
    @Override
    public synchronized int getRowCount() {
        if (strategy != null)
            return strategy.getPositions().size();
        else
            return 0;
    }

    @Override
    public int getColumnCount() {
        return 8;
    }

    @Override
    public synchronized Object getValueAt(int rowIndex, int columnIndex) {
        if (strategy != null)
        {
            Position position = strategy.getPositions().get(rowIndex);
            switch (columnIndex)
            {
                case 0:
                    return position.getPositionType();
                case 1:
                    return position.getTicker();
                case 2:
                    return position.getPrice();
                case 3:
                    return position.getShares();
                case 4:
                    return position.getExposure();
                case 5:
                    return position.getTargetAllocationPercent();
                case 6:
                    return position.getActualAllocationPercent();
                case 7: 
                    return position.getDeviation();
                default:
                    return null;
            }
        }
        else
            return null;
    }
    
    /**
     * Gets whether the position at the specified row is in an alerted state.
     * @param modelRowIndex row index
     * @return whether the position is alerted.
     */
    public synchronized boolean isRowAlerted(int modelRowIndex)
    {
        if (strategy != null && modelRowIndex < strategy.getPositions().size()) {
            Position position = strategy.getPositions().get(modelRowIndex);
            return position.isAlerted();
        }
        else {
            return false;
        }
    }
    
}
