/* 
 * Copyright 2013 ScaleOut Software, Inc.
 * 
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

package com.scaleoutsoftware.demos.common;

import java.io.Serializable;
import java.util.BitSet;

/**
 * A summary of currently alerted strategies.
 * 
 * Invocation operations return objects of this type to efficiently indicate which strategies
 * in the StateServer store are alerted (that is, their positions fall too far outside desired
 * core/hedge allocations).
 * 
 * A single instance of this class is what's ultimately returned to the caller of the NamedCache.Invoke()
 * method that initiates the PMI operation. The PMI operation merges all returned summaries from all 
 * strategy evaluations and combines them into a single value that is returned to the originating 
 * NamedCache.Invoke() caller. This merging operation is effectively the "reduce" step of the PMI 
 * feature's map/reduce style behavior. Thus, we provide a merge() method on this class that allows us
 * to combine two of these result instances into one.
 */
public class AlertSummary implements Serializable {
    private static final long serialVersionUID = 1L;
    private BitSet alertedStrategies = new BitSet();
    private int totalStrategyCount = 0;
    private int totalPositionsEvaluated = 0;
    private long elapsedEvaluationTime = 0;
    
    /**
     * Constructs a newly allocated summary object.
     */
    public AlertSummary() {
     
    }
     
    /**
     * Constructs a newly allocated summary object with a single strategy set
     * to the provided alert state.
     * 
     * @param strategyId                 integer identifier of the strategy
     * @param alerted            indicates whether the strategy is in an alerted state
     * @param positionsEvaluated number of positions that were evaluated in the strategy
     */
    public AlertSummary(int strategyId, boolean alerted, int positionsEvaluated) {
        this.alertedStrategies.set(strategyId, alerted);
        this.totalStrategyCount = 1;
        this.totalPositionsEvaluated = positionsEvaluated;
    }
    
    /**
     * Merges another summary into this instance.
     * 
     * ScaleOut's PMI engine reduces the results of all Invokable.eval() calls into a single result
     * that is returned to the original NamedCache.Invoke() caller. This method facilitates combining
     * two instances.
     * 
     * @param other  the summary to be merged into this instance
     */
    public void merge(AlertSummary other)
    {
        if (other != null)
        {
            // A bitwise OR operation on a BitSet will cause it to grow as needed.
            this.alertedStrategies.or(other.getAlertedStrategies());
            
            totalStrategyCount += other.getTotalStrategyCount();
            totalPositionsEvaluated += other.getTotalPositionsEvaluated();
        }
        
    }

    /**
     * @return A BitSet summarizing alert status for strategies in the data grid.
     * 
     * If an element at a given index in the returned collection is true
     * then the strategy with an ID corresponding to that index is in an
     * alerted state.
     */
    public BitSet getAlertedStrategies() {
        return alertedStrategies;
    }
    
    /**
     * Returns the total number of strategies that were evaluated in the data grid, regardless of alert status.
     * @return  strategy count as integer
     */
    public int getTotalStrategyCount() {
        return totalStrategyCount;
    }

    /**
     * Gets the total number of positions that were evaluated during during the last
     * strategy update invocation
     * @return the totalPositionsEvaluated
     */
    public int getTotalPositionsEvaluated() {
        return totalPositionsEvaluated;
    }

    /**
     * @return the elapsed evaluation time in milliseconds
     */
    public long getElapsedEvaluationTime() {
        return elapsedEvaluationTime;
    }

    /**
     * @param timeInMilliseconds the elapsed evaluation time to set (milliseconds)
     */
    public void setElapsedEvaluationTime(long timeInMilliseconds) {
        this.elapsedEvaluationTime = timeInMilliseconds;
    }
    
}
