/* 
 * Copyright 2013 ScaleOut Software, Inc.
 * 
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

package com.scaleoutsoftware.demos.common;

import java.io.Serializable;
import java.math.BigDecimal;

/**
 * Represents a position that can be held in a Strategy.
 *
 * A builder pattern is used to construct Position instances using
 * a static member class (Position.Builder).
 */
public class Position implements Serializable {
    private static final long serialVersionUID = 1L;
    private final String ticker;
    private final PositionType positionType;
    private BigDecimal price;
    private BigDecimal shares;
    private BigDecimal targetAllocationPercent;
    private BigDecimal actualAllocationPercent;
    private boolean alerted;

    /**
     * Builds a Position instance.
     */
    public static class Builder {
        // Required parameters
        private final String ticker;
        private final PositionType positionType;

        // Optional parameters - initialized to defaults
        private BigDecimal price = BigDecimal.ZERO;
        private BigDecimal shares = BigDecimal.ZERO;
        private BigDecimal targetAllocationPercent = BigDecimal.ZERO;
        private BigDecimal actualAllocationPercent = BigDecimal.ZERO;
        private boolean alerted = false;

        /**
         * Strategy Builder constructor
         * @param ticker       required ticker symbol
         * @param positionType required position type (core or hedge)
         */
        public Builder(String ticker, PositionType positionType) {
            this.ticker = ticker;
            this.positionType = positionType;
        }
        
        // Builder setters:

        public Builder price(BigDecimal val)
        {price = val; return this;}

        public Builder shares(BigDecimal val)
        {shares = val; return this;}

        public Builder targetAllocationPercent(BigDecimal val)
        {targetAllocationPercent = val; return this;}

        public Builder actualAllocationPercent(BigDecimal val)
        {actualAllocationPercent = val; return this;}

        public Builder alerted(boolean val)
        {alerted = val; return this;}

        public Position build() {
            return new Position(this);
        }

    }

    private Position(Builder builder) {
        ticker = builder.ticker;
        positionType = builder.positionType;
        price = builder.price;
        shares = builder.shares;
        targetAllocationPercent = builder.targetAllocationPercent;
        actualAllocationPercent = builder.actualAllocationPercent;
        alerted = builder.alerted;
    }

    /**
     * @return the ticker symbol identifying the security held in this position
     */
    public String getTicker() {
        return ticker;
    }
    
    /**
     * @return the type of position (core or hedge) that's held in the strategy
     */
    public PositionType getPositionType() {
        return positionType;
    }
    
    /**
     * @return the latest known price of the security held in this position
     */
    public BigDecimal getPrice() {
        return price;
    }

    /**
     * @param price  sets the latest price for the security associated with this position.
     */
    public void setPrice(BigDecimal price) {
        this.price = price;
    }
    
    /**
     * @return the number of shares held
     */
    public BigDecimal getShares() {
        return shares;
    }

    /**
     * @param shares  sets the number of shares held in this position
     */
    public void setShares(BigDecimal shares) {
        this.shares = shares;
    }
    
    /**
     * @return price * shares held
     */
    public BigDecimal getExposure() {
        BigDecimal exposure = getPrice().multiply(this.getShares());
        return exposure.setScale(5, BigDecimal.ROUND_HALF_UP);
    }
    
    /**
     * @return  the desired percentage of total strategy value that this position should hold
     */
    public BigDecimal getTargetAllocationPercent() {
        return targetAllocationPercent;
    }

    /**
     * @param targetAllocationPercent sets the desired percentage of total strategy value that this position should hold
     */
    public void setTargetAllocationPercent(BigDecimal targetAllocationPercent) {
        this.targetAllocationPercent = targetAllocationPercent;
    }

    /**
     * @return  the actual percentage of total strategy value held by this position
     */
    public BigDecimal getActualAllocationPercent() {
        return actualAllocationPercent;
    }

    /**
     * @param actualAllocationPercent  sets the actual percentage of total strategy value held by this position
     */
    public void setActualAllocationPercent(BigDecimal actualAllocationPercent) {
        this.actualAllocationPercent = actualAllocationPercent;
    }

    /**
     * @return the difference between the actual percentage the position occupies in 
     *         the strategy vs. the desired percentage.
     */
    public BigDecimal getDeviation() {
        return getActualAllocationPercent().subtract(getTargetAllocationPercent());
    }

    /**
     * @return whether this position exceeds the targeted allocation by such a large margin
     *         that it is now in an alerted state
     */
    public boolean isAlerted() {
        return alerted;
    }

    /**
     * 
     * @param alerted  whether this position is in an alerted state
     */
    public void setAlerted(boolean alerted) {
        this.alerted = alerted;
    }
}
