/* 
 * Copyright 2013 ScaleOut Software, Inc.
 * 
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

package com.scaleoutsoftware.demos.common;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Calendar;

/**
 * Security class containing a security's latest known price.
 */
public final class Security implements Serializable, Cloneable {
    private static final long serialVersionUID = 1L;
    private final String ticker;
    private BigDecimal latestPrice;
    private Calendar latestPriceTime;

    /**
     * Constructor
     * @param ticker security identifier
     */
    public Security(String ticker) {
        this.ticker = ticker;
    }

    /**
     * Constructor
     * @param ticker          security identifier
     * @param latestPrice     current price for the security
     * @param latestPriceTime time of the current price quote
     */
    public Security(String ticker, BigDecimal latestPrice, Calendar latestPriceTime)
    {
        this.ticker = ticker;
        this.latestPrice = latestPrice;
        this.latestPriceTime = latestPriceTime;
    }

    /**
     * Create a deep copy of this security object
     * @return clone of the security object
     */
    @Override public Security clone()
    {
        // all members are immutable types except for the latestPriceTime.
        return new Security(this.ticker, this.latestPrice, (Calendar)this.latestPriceTime.clone());
    }

    /**
     * Update the price of this security.
     * The LatestPriceTime for this security is automatically updated to the current time.
     * 
     * @param latestPrice new price
     */
    public void updatePrice(BigDecimal latestPrice) {
        this.latestPrice = latestPrice;
        this.latestPriceTime = Calendar.getInstance();
    }

    /**
     * Identifier for this security.
     * @return stock symbol
     */
    public String getTicker() {
        return ticker;
    }

    /**
     * Get the current price for the security
     * @return current price
     */
    public BigDecimal getLatestPrice() {
        return latestPrice;
    }

    /**
     * @return time of the current price quote
     */
    public Calendar getLatestPriceTime() {
        return latestPriceTime;
    }

}
