/* 
 * Copyright 2013 ScaleOut Software, Inc.
 * 
 * LICENSE AND DISCLAIMER
 * ----------------------
 * This material contains sample programming source code ("Sample Code").
 * ScaleOut Software, Inc. (SSI) grants you a nonexclusive license to compile, 
 * link, run, display, reproduce, and prepare derivative works of 
 * this Sample Code.  The Sample Code has not been thoroughly
 * tested under all conditions.  SSI, therefore, does not guarantee
 * or imply its reliability, serviceability, or function. SSI
 * provides no support services for the Sample Code.
 *
 * All Sample Code contained herein is provided to you "AS IS" without
 * any warranties of any kind. THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGMENT ARE EXPRESSLY
 * DISCLAIMED.  SOME JURISDICTIONS DO NOT ALLOW THE EXCLUSION OF IMPLIED
 * WARRANTIES, SO THE ABOVE EXCLUSIONS MAY NOT APPLY TO YOU.  IN NO 
 * EVENT WILL SSI BE LIABLE TO ANY PARTY FOR ANY DIRECT, INDIRECT, 
 * SPECIAL OR OTHER CONSEQUENTIAL DAMAGES FOR ANY USE OF THE SAMPLE CODE
 * INCLUDING, WITHOUT LIMITATION, ANY LOST PROFITS, BUSINESS 
 * INTERRUPTION, LOSS OF PROGRAMS OR OTHER DATA ON YOUR INFORMATION
 * HANDLING SYSTEM OR OTHERWISE, EVEN IF WE ARE EXPRESSLY ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGES.
 */

package com.scaleoutsoftware.demos.marketsimulator;

import com.scaleoutsoftware.demos.common.*;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.HashMap;
import java.util.Random;

/**
 * Simulates a market feed that periodically changes market prices on a collection of securities.
 * 
 * Clients of this runnable class should call getSnapshot() to retrieve a deep copy of the latest
 * market prices for all securities.
 */
public class ExchangeSimulator implements Runnable {
    // Maximum % that a simulated price can change:
    private static final float MAX_PRICE_CHANGE_PERCENT = 1.0f;
    
    private volatile int priceUpdatesPerSec;
    private volatile HashMap<String, Security> latestPrices;

    /**
     * Constructor
     * @param priceUpdatesPerSec the rate of incoming security price updates to simulate
     * @param securities         the collection of securities in the simulated exchange. The provided
     *                           collection will not be modified (a deep copy will be made).
     */
    public ExchangeSimulator(int priceUpdatesPerSec, HashMap<String, Security> securities) {
        this.priceUpdatesPerSec = priceUpdatesPerSec;

        latestPrices = new HashMap<String, Security>(securities.size());
        for (Security security : securities.values())
            latestPrices.put(security.getTicker(), security.clone());
    }

    /**
     * Price update worker. Modifies random securities so that their price changes by a random
     * percentage.
     */
    @Override
    public void run() {
        Random rand = new Random();

        // converting the map of securities to an array in order to ease random selection:
        Security[] securities = latestPrices.values().toArray(new Security[latestPrices.size()]);

        while (!Thread.currentThread().isInterrupted())
        {
            long startTime = System.nanoTime();
            for (int i = 0; i < priceUpdatesPerSec; ++i)
            {
                Security security = securities[rand.nextInt(latestPrices.size())];
                BigDecimal oldPrice = security.getLatestPrice();

                // Multiply max allowed by 100, convert rand to a float, then divide by 100:
                float percentChange = ((float)rand.nextInt((int)(MAX_PRICE_CHANGE_PERCENT * 100) )) / 100.0f;

                float priceMultiplier = percentChange / 100.0f;
                // Decide whether the price goes up or down:
                priceMultiplier = rand.nextBoolean() ? priceMultiplier : (priceMultiplier * -1f);

                BigDecimal newPrice = oldPrice.add(oldPrice.multiply(BigDecimal.valueOf(priceMultiplier)));
                newPrice = newPrice.setScale(5, RoundingMode.HALF_UP);

                synchronized (latestPrices) {
                    security.updatePrice(newPrice);
                }
            }

            long elapsedTimeMillis = (System.nanoTime() - startTime) / 1000000;
            if (elapsedTimeMillis < 1000)
            {
                // The simulated market feed produced the desired updates/sec with time to spare. Pause
                // for the remainder of the 1-second interval. Empirically, it only takes a few milliseconds
                // to simulate a thousand incoming prices, so this thread spends most of its time in
                // this wait call.
                long sleepTime = 1000 - elapsedTimeMillis;
                try {
                    synchronized (this) {
                        this.wait(sleepTime);
                    }
                } catch (InterruptedException ex) {
                    return;
                }
            }
            else
            {
                System.out.format("Warning: Simulated market feed was unable to generate %d prices in one second. Last interval took %d milliseconds.\n", priceUpdatesPerSec, elapsedTimeMillis);
            }
        }

    }

    /**
     * Returns the latest security prices in this exchange. A deep copy is returned in case multiple
     * threads are reading/modifying the contents of the ExchangeSimulator's internal collection.
     * 
     * @return a collection of security prices
     */
    public HashMap<String, Security> getPriceSnapshot() {
        HashMap<String, Security> deepCopy;

        synchronized (latestPrices) {
            deepCopy = new HashMap<String, Security>(latestPrices.size());
            for (Security security : latestPrices.values())
                deepCopy.put(security.getTicker(), security.clone());
        }
        return deepCopy;
    }
}
